@@ -1,26 +1,24 @@
-from cloudmesh.common.JobSet import JobSet
-import os
 import textwrap
-from cloudmesh.common.dotdict import dotdict
-from cloudmesh.common.Tabulate import Printer
-from pprint import pprint
+
+from cloudmesh.common.JobSet import JobSet
+from cloudmesh.common.Shell import Shell
 from cloudmesh.common.console import Console
 from cloudmesh.common.parameter import Parameter
-from cloudmesh.common.Shell import Shell
+
 
 class JobMultiHostScript:
     '''
     The JobMultiHostScript is a simple mechanism to run a number of commands formylated
     in a script in parallel over the hosts provided. The script is interpreted line
     by line and does not support multi line commands at this time. (Not difficult
-    to implement when looking at \ at the end of a line.)
+    to implement when looking at \\ at the end of a line.)
 
     Static method::
         script = """
           # This is a comment
           pwd     # tag: pwd
           uname -a
-        """;
+        """
 
         hosts = Parameter.expand("purple[01-02]")
         result = JobMultiHostScript.execute(script, "script_name", hosts)
@@ -61,11 +59,10 @@ class JobMultiHostScript:
 
                 # Execute jobSet for each host in parallel
                 for host in self.hosts:
-                    job.add({"name": host, "host": host, "command": line});
+                    job.add({"name": host, "host": host, "command": line})
                 job.run(parallel=len(self.hosts))
                 job.Print()
 
-
     @staticmethod
     def execute(script, name="script", hosts=None, executor=JobSet.ssh,
                 beginLine=None, endLine=None):
@@ -110,6 +107,7 @@ class JobMultiHostScript:
                 Console.error("The script is not defined, found None as content")
             return
 
+
 if __name__ == '__main__':
     script = """
     # This is a comment
@@ -119,7 +117,7 @@ if __name__ == '__main__':
 
     # Task: uname
     uname -a
-    """;
+    """
 
     hosts = Parameter.expand("purple[01-02]")
     result = JobMultiHostScript.execute(script, "script_name", hosts,
