@@ -4,26 +4,30 @@ that this method requires that the command be completely execute before the
 output is returned. FOr many activities in cloudmesh this is sufficient.
 
 """
-
 import errno
 import glob
 import os
 import platform as os_platform
+import requests
 import shlex
 import shutil
 import subprocess
 import sys
+import textwrap
 import zipfile
 from distutils.spawn import find_executable
 from pathlib import Path
 from pipes import quote
 from sys import platform
+from tqdm import tqdm
 
 import psutil
 from cloudmesh.common.StopWatch import StopWatch
 from cloudmesh.common.console import Console
 from cloudmesh.common.dotdict import dotdict
-from cloudmesh.common.util import path_expand, readfile
+from cloudmesh.common.systeminfo import get_platform
+from cloudmesh.common.util import path_expand
+from cloudmesh.common.util import readfile
 
 
 # from functools import wraps
@@ -46,10 +50,20 @@ from cloudmesh.common.util import path_expand, readfile
 #        f(args)
 #    return new_f
 
+def windows_not_supported(f):
+    def wrapper(*args, **kwargs):
+        host = get_platform()
+        if host == "windows":
+            Console.error("Not supported on windows")
+            return ""
+        else:
+            return f(*args, **kwargs)
+
+    return wrapper
+
 def NotImplementedInWindows():
     if sys.platform == "win32":
-        Console.error(f"The method {__name__} is not implemented in Windows,"
-                      " please implement, and/or submit an issue.")
+        Console.error(f"The method {__name__} is not implemented in Windows.")
         sys.exit()
 
 
@@ -80,11 +94,11 @@ class Brew(object):
 class Pip(object):
     @classmethod
     def install(cls, name):
-        r = Shell.pip("install", "esptool")
-        if "already satisfied: esptool" in r:
+        r = Shell.pip("install", name)
+        if f"already satisfied: {name}" in r:
             print(name, "... already installed")
             Console.ok(r)
-        elif "Successfully installed esptool":
+        elif f"Successfully installed {name}":
             print(name, "... installed")
             Console.ok(r)
         else:
@@ -94,8 +108,8 @@ class Pip(object):
 
 class SubprocessError(Exception):
     """
-    Manages the formatting of the error and stdout. 
-    THis command should not be directly called. Instead use SHell
+    Manages the formatting of the error and stdout.
+    This command should not be directly called. Instead use Shell
     """
 
     def __init__(self, cmd, returncode, stderr, stdout):
@@ -116,10 +130,10 @@ class SubprocessError(Exception):
         def indent(lines, amount, ch=' '):
             """
             indent the lines by multiples of ch
-            :param lines: 
-            :param amount: 
-            :param ch: 
-            :return: 
+            :param lines:
+            :param amount:
+            :param ch:
+            :return:
             """
             padding = amount * ch
             return padding + ('\n' + padding).join(lines.split('\n'))
@@ -151,7 +165,7 @@ class Subprocess(object):
         :param cwd: the directory in which to execute the command
         :param stderr: the pipe for stderror
         :param stdout: the pipe for the stdoutput
-        :param env: 
+        :param env:
         """
         Console.debug_msg('Running cmd: {}'.format(' '.join(map(quote, cmd))))
 
@@ -170,7 +184,7 @@ class Subprocess(object):
 
 class Shell(object):
     """
-    The shell class allowing us to conveniently access many operating system commands. 
+    The shell class allowing us to conveniently access many operating system commands.
     TODO: This works well on Linux and OSX, but has not been tested much on Windows
     """
 
@@ -202,11 +216,34 @@ class Shell(object):
     # or do something more simple
     #
     # ls = cls.execute('cmd', args...)
+    @staticmethod
+    def timezone(default="America/Indiana/Indianapolis"):
+        # BUG we need to be able to pass the default from the cmdline
+        host = get_platform()
+        if host == "windows":
+            return default
+        else:
+            # result = Shell.run("ls -l /etc/localtime").strip().split("/")
+            try:
+                result = Shell.run("ls -l /etc/localtime").strip().split("zoneinfo")[1][1:]
+                return result
+            except IndexError as e:
+                return default
+
+    @staticmethod
+    @windows_not_supported
+    def locale():
+        try:
+            result = Shell.run('locale').split('\n')[0].split('_')[1].split('.')[0].lower()
+            return result
+        except IndexError as e:
+            Console.warning('Could not determine locale. Defaulting to "us"')
+            return 'us'
 
     @staticmethod
     def run_timed(label, command, encoding=None, service=None):
         """
-        runs teh command and uses the StopWatch to time it
+        runs the command and uses the StopWatch to time it
         :param label: name of the StopWatch
         :param command: the command to be executed
         :param encoding: the encoding
@@ -330,10 +367,12 @@ class Shell(object):
 
         if cwd is None:
             cwd = os.getcwd()
+        # noinspection PyPep8
         try:
             if shell:
                 if platform.lower() == 'win32':
                     import ctypes
+
                     class disable_file_system_redirection:
                         _disable = ctypes.windll.kernel32.Wow64DisableWow64FsRedirection
                         _revert = ctypes.windll.kernel32.Wow64RevertWow64FsRedirection
@@ -374,6 +413,22 @@ class Shell(object):
             result = result.strip().decode()
         return result
 
+    @staticmethod
+    def oneline(script):
+        """
+        converts a script to one line command.
+        THis is useful to run a single ssh command and pass a one line script.
+
+        :param script:
+        :return:
+        """
+        return " && ".join(textwrap.dedent(script).strip().splitlines())
+
+    @staticmethod
+    def is_root():
+        username = subprocess.getoutput("whoami")
+        return username == "root"
+
     @staticmethod
     def rmdir(top, verbose=False):
 
@@ -407,7 +462,6 @@ class Shell(object):
         else:
             os.system("python -m webbrowser -t {file}".format(**data))
 
-
     @staticmethod
     def terminal_title(name):
         return f'echo -n -e \"\033]0;{name}\007\"'
@@ -415,7 +469,7 @@ class Shell(object):
     @classmethod
     def terminal(cls, command='pwd', title=None):
         # title nameing not implemented
-        print (platform)
+        print(platform)
         if platform == 'darwin':
             label = Shell.terminal_title(title)
 
@@ -423,13 +477,14 @@ class Shell(object):
                 f"osascript -e 'tell application \"Terminal\" to do script \"{command}\"'"
             )
         elif platform == "linux":  # for ubuntu running gnome
-            os.system(f"gnome-terminal -e \"bash -c \'{command}; exec $SHELL\'\"")
-
+            dist = os_platform.linux_distribution()[0]
+            linux_apps = {'ubuntu': 'gnome-terminal', 'debian':'lxterminal'}
+            os.system(f"{linux_apps[dist]} -e \"bash -c \'{command}; exec $SHELL\'\"")
 
         elif platform == "win32":
             if Path.is_dir(Path(r"C:\Program Files\Git")):
                 subprocess.Popen([r"C:\Program Files\Git\git-bash.exe",
-                                       "-c", f"{command}"])
+                                  "-c", f"{command}"])
             else:
                 Console.error("Git bash is not found, please make sure it "
                               "is installed.")
@@ -483,7 +538,7 @@ class Shell(object):
     def ls(cls, match="."):
         """
         executes ls with the given arguments
-        :param args: 
+        :param args:
         :return: list
         """
         d = glob.glob(match)
@@ -524,8 +579,8 @@ class Shell(object):
     def bash(cls, *args):
         """
         executes bash with the given arguments
-        :param args: 
-        :return: 
+        :param args:
+        :return:
         """
         return cls.execute('bash', args)
 
@@ -534,8 +589,8 @@ class Shell(object):
     def brew(cls, *args):
         """
         executes bash with the given arguments
-        :param args: 
-        :return: 
+        :param args:
+        :return:
         """
         NotImplementedInWindows()
         return cls.execute('brew', args)
@@ -545,8 +600,8 @@ class Shell(object):
     def cat(cls, *args):
         """
         executes cat with the given arguments
-        :param args: 
-        :return: 
+        :param args:
+        :return:
         """
         NotImplementedInWindows()
         # TODO: replace with file read and reading the content. We need to deal
@@ -558,8 +613,8 @@ class Shell(object):
     def git(cls, *args):
         """
         executes git with the given arguments
-        :param args: 
-        :return: 
+        :param args:
+        :return:
         """
         NotImplementedInWindows()
         return cls.execute('git', args)
@@ -569,8 +624,8 @@ class Shell(object):
     def VBoxManage(cls, *args):
         """
         executes VboxManage with the given arguments
-        :param args: 
-        :return: 
+        :param args:
+        :return:
         """
 
         if platform == "darwin":
@@ -583,8 +638,8 @@ class Shell(object):
     def blockdiag(cls, *args):
         """
         executes blockdiag with the given arguments
-        :param args: 
-        :return: 
+        :param args:
+        :return:
         """
         return cls.execute('blockdiag', args)
 
@@ -592,8 +647,8 @@ class Shell(object):
     def cm(cls, *args):
         """
         executes cm with the given arguments
-        :param args: 
-        :return: 
+        :param args:
+        :return:
         """
         return cls.execute('cm', args)
 
@@ -620,8 +675,8 @@ class Shell(object):
     def head(cls, *args):
         """
         executes head with the given arguments
-        :param args: 
-        :return: 
+        :param args:
+        :return:
         """
         NotImplementedInWindows()
         # TODO: reimplement with readlines
@@ -631,8 +686,8 @@ class Shell(object):
     def keystone(cls, *args):
         """
         executes keystone with the given arguments
-        :param args: 
-        :return: 
+        :param args:
+        :return:
         """
         return cls.execute('keystone', args)
 
@@ -656,19 +711,76 @@ class Shell(object):
     def kill(cls, *args):
         """
         executes kill with the given arguments
-        :param args: 
-        :return: 
+        :param args:
+        :return:
         """
         NotImplementedInWindows()
         # TODO: use tasklisk, compare to linux
         return cls.execute('kill', args)
 
+    @classmethod
+    def download(cls, source, destination, force=False, provider=None, chunk_size=128):
+        """
+        Given a source url and a destination filename, download the file at the source url
+        to the destination.
+
+        If provider is None, the request lib is used
+        If provider is 'system', wget, curl, and requests lib are attempted in that order
+        """
+        destination = path_expand(destination)
+
+        if os.path.exists(destination) and not force:
+            return destination
+
+        if provider == 'system':
+            # First try wget
+            wget_return = os.system(f'wget -O {destination} {source}')
+            if wget_return == 0:
+                Console.info('Used wget')
+                return destination
+            # Then curl
+            curl_return = os.system(f'curl -L -o {destination} {source}')
+            if curl_return == 0:
+                Console.info('Used curl')
+                return destination
+        # Default is requests lib. If provider is None, or if provider == 'system'
+        # but wget and curl fail, default here
+        r = requests.get(source, stream=True, allow_redirects=True)
+        total_size = int(r.headers.get('content-length'))
+
+        with open(destination, 'wb') as fd:
+            with tqdm(total=total_size, unit="B",
+               unit_scale=True, desc=destination, initial=0, ascii=True) as pbar:
+                for chunk in r.iter_content(chunk_size=chunk_size):
+                    fd.write(chunk)
+                    pbar.update(len(chunk))
+
+            return destination
+
+    @classmethod
+    def mount(cls, *args):
+        """
+        mounts a given mountpoint to a file
+        :param args:
+        :return:
+        """
+        return cls.execute('mount', args)
+
+    @classmethod
+    def umount(cls, *args):
+        """
+        umounts a given mountpoint to a file
+        :param args:
+        :return:
+        """
+        return cls.execute('umount', args)
+
     @classmethod
     def nova(cls, *args):
         """
         executes nova with the given arguments
-        :param args: 
-        :return: 
+        :param args:
+        :return:
         """
         return cls.execute('nova', args)
 
@@ -676,9 +788,9 @@ class Shell(object):
     def ping(cls, host=None, count=1):
         """
         execute ping
-        :param host: the host to ping 
+        :param host: the host to ping
         :param count: the number of pings
-        :return: 
+        :return:
         """
         option = '-n' if platform == 'windows' else '-c'
         return cls.execute('ping',
@@ -690,8 +802,8 @@ class Shell(object):
     def pwd(cls, *args):
         """
         executes pwd with the given arguments
-        :param args: 
-        :return: 
+        :param args:
+        :return:
         """
         return os.getcwd()
 
@@ -699,8 +811,8 @@ class Shell(object):
     def rackdiag(cls, *args):
         """
         executes rackdiag with the given arguments
-        :param args: 
-        :return: 
+        :param args:
+        :return:
         """
         return cls.execute('rackdiag', args)
 
@@ -708,9 +820,9 @@ class Shell(object):
     # @NotImplementedInWindows
     def rm(cls, location):
         """
-        executes rm with the given arguments
-        :param args: 
-        :return: 
+        executes rm tree with the given arguments
+        :param args:
+        :return:
         """
         shutil.rmtree(path_expand(location))
 
@@ -718,8 +830,8 @@ class Shell(object):
     def rsync(cls, *args):
         """
         executes rsync with the given arguments
-        :param args: 
-        :return: 
+        :param args:
+        :return:
         """
         return cls.execute('rsync', args)
 
@@ -727,8 +839,8 @@ class Shell(object):
     def scp(cls, *args):
         """
         executes scp with the given arguments
-        :param args: 
-        :return: 
+        :param args:
+        :return:
         """
         return cls.execute('scp', args)
 
@@ -737,8 +849,8 @@ class Shell(object):
     def sort(cls, *args):
         """
         executes sort with the given arguments
-        :param args: 
-        :return: 
+        :param args:
+        :return:
         """
         NotImplementedInWindows()
         # TODO: https://superuser.com/questions/1316317/is-there-a-windows-equivalent-to-the-unix-uniq
@@ -748,8 +860,8 @@ class Shell(object):
     def sh(cls, *args):
         """
         executes sh with the given arguments
-        :param args: 
-        :return: 
+        :param args:
+        :return:
         """
         return cls.execute('sh', args)
 
@@ -757,8 +869,8 @@ class Shell(object):
     def ssh(cls, *args):
         """
         executes ssh with the given arguments
-        :param args: 
-        :return: 
+        :param args:
+        :return:
         """
         return cls.execute('ssh', args)
 
@@ -767,8 +879,8 @@ class Shell(object):
     def sudo(cls, *args):
         """
         executes sudo with the given arguments
-        :param args: 
-        :return: 
+        :param args:
+        :return:
         """
         NotImplementedInWindows()
         # TODO: https://stackoverflow.com/questions/9652720/how-to-run-sudo-command-in-windows
@@ -780,18 +892,18 @@ class Shell(object):
         """
         executes tail with the given arguments
         :param args:
-        :return: 
+        :return:
         """
         NotImplementedInWindows()
-        # TODO: implement with realines on a file.
+        # TODO: implement with readlines on a file.
         return cls.execute('tail', args)
 
     @classmethod
     def vagrant(cls, *args):
         """
         executes vagrant with the given arguments
-        :param args: 
-        :return: 
+        :param args:
+        :return:
         """
         return cls.execute('vagrant', args, shell=True)
 
@@ -799,8 +911,8 @@ class Shell(object):
     def pandoc(cls, *args):
         """
         executes vagrant with the given arguments
-        :param args: 
-        :return: 
+        :param args:
+        :return:
         """
         return cls.execute('pandoc', args)
 
@@ -808,8 +920,8 @@ class Shell(object):
     def mongod(cls, *args):
         """
         executes mongod with the given arguments
-        :param args: 
-        :return: 
+        :param args:
+        :return:
         """
         return cls.execute('mongod', args)
 
@@ -818,8 +930,8 @@ class Shell(object):
     def dialog(cls, *args):
         """
         executes dialof with the given arguments
-        :param args: 
-        :return: 
+        :param args:
+        :return:
         """
         NotImplementedInWindows()
         return cls.execute('dialog', args)
@@ -828,8 +940,8 @@ class Shell(object):
     def pip(cls, *args):
         """
         executes pip with the given arguments
-        :param args: 
-        :return: 
+        :param args:
+        :return:
         """
         return cls.execute('pip', args)
 
@@ -874,9 +986,9 @@ class Shell(object):
     def remove_line_with(cls, lines, what):
         """
         returns all lines that do not contain what
-        :param lines: 
-        :param what: 
-        :return: 
+        :param lines:
+        :param what:
+        :return:
         """
         result = []
         for line in lines:
@@ -888,9 +1000,9 @@ class Shell(object):
     def find_lines_with(cls, lines, what):
         """
         returns all lines that contain what
-        :param lines: 
-        :param what: 
-        :return: 
+        :param lines:
+        :param what:
+        :return:
         """
         result = []
         for line in lines:
@@ -914,7 +1026,6 @@ class Shell(object):
                 result = result + [line]
         return result
 
-
     @classmethod
     def find_lines_between(cls, lines, what_from, what_to):
         """
@@ -976,8 +1087,8 @@ class Shell(object):
     def command_exists(cls, name):
         """
         returns True if the command exists
-        :param name: 
-        :return: 
+        :param name:
+        :return:
         """
         return cls.which(name) is not None
 
@@ -1001,7 +1112,6 @@ class Shell(object):
     def cms(command, encoding='utf-8'):
         return Shell.run("cms " + command, encoding=encoding)
 
-
     @classmethod
     def check_python(cls):
         """
@@ -1030,8 +1140,7 @@ class Shell(object):
 
         elif python_version[0] == 3:
 
-            if (python_version[0] == 3) and (python_version[1] >= 7) and (
-                python_version[2] >= 0):
+            if (python_version[0] == 3) and (python_version[1] >= 7) and (python_version[2] >= 0):
 
                 print(
                     "You are running a supported version of python: {:}".format(
@@ -1054,13 +1163,12 @@ class Shell(object):
             print("         We recommend you update your pip  with \n")
             print("             pip install -U pip\n")
 
-
     @classmethod
     def mkdir(cls, directory):
         """
         creates a directory with all its parents in ots name
         :param directory: the path of the directory
-        :return: 
+        :return:
         """
         directory = path_expand(directory)
         try:
@@ -1178,7 +1286,7 @@ class Shell(object):
 def main():
     """
     a test that should actually be added into a pytest
-    :return: 
+    :return:
     """
 
     print(Shell.terminal_type())
